@@ -38,6 +38,7 @@
 #include "module-filter-apply-symdef.h"
 
 #define PA_PROP_FILTER_APPLY_MOVING "filter.apply.moving"
+#define PA_PROP_MDM_AUTO_FILTERED   "module-device-manager.auto_filtered"
 
 PA_MODULE_AUTHOR("Colin Guthrie");
 PA_MODULE_DESCRIPTION("Load filter sinks automatically when needed");
@@ -65,17 +66,10 @@ struct filter {
 struct userdata {
     pa_core *core;
     pa_hashmap *filters;
-    pa_hook_slot
-        *sink_input_put_slot,
-        *sink_input_move_finish_slot,
-        *sink_input_proplist_slot,
-        *sink_input_unlink_slot,
-        *sink_unlink_slot,
-        *source_output_put_slot,
-        *source_output_move_finish_slot,
-        *source_output_proplist_slot,
-        *source_output_unlink_slot,
-        *source_unlink_slot;
+    /* Keep track of streams we're managing PA_PROP_MDM_AUTO_FILTERED on, we're
+     * only maintaining membership, so key and value are just the
+     * pa_sink_input/pa_source_output. */
+    pa_hashmap *mdm_ignored_inputs, *mdm_ignored_outputs;
     bool autoclean;
     pa_time_event *housekeeping_time_event;
 };
@@ -135,7 +129,7 @@ static const char* should_filter(pa_object *o, bool is_sink_input) {
     else
         pl = PA_SOURCE_OUTPUT(o)->proplist;
 
-    /* If the stream doesn't what any filter, then let it be. */
+    /* If the stream doesn't want any filter, then let it be. */
     if ((apply = pa_proplist_gets(pl, PA_PROP_FILTER_APPLY)) && !pa_streq(apply, "")) {
         const char* suppress = pa_proplist_gets(pl, PA_PROP_FILTER_SUPPRESS);
 
@@ -281,14 +275,20 @@ static void trigger_housekeeping(struct userdata *u) {
     u->housekeeping_time_event = pa_core_rttime_new(u->core, pa_rtclock_now() + HOUSEKEEPING_INTERVAL, housekeeping_time_callback, u);
 }
 
-static int do_move(pa_object *obj, pa_object *parent, bool restore, bool is_input) {
-    if (is_input)
-        return pa_sink_input_move_to(PA_SINK_INPUT(obj), PA_SINK(parent), restore);
-    else
-        return pa_source_output_move_to(PA_SOURCE_OUTPUT(obj), PA_SOURCE(parent), restore);
+static int do_move(struct userdata *u, pa_object *obj, pa_object *parent, bool is_input) {
+    /* Keep track of objects that we've marked for module-device-manager to ignore */
+    pa_hashmap_put(is_input ? u->mdm_ignored_inputs : u->mdm_ignored_outputs, obj, obj);
+
+    if (is_input) {
+        pa_sink_input_set_property(PA_SINK_INPUT(obj), PA_PROP_MDM_AUTO_FILTERED, "1");
+        return pa_sink_input_move_to(PA_SINK_INPUT(obj), PA_SINK(parent), false);
+    } else {
+        pa_source_output_set_property(PA_SOURCE_OUTPUT(obj), PA_PROP_MDM_AUTO_FILTERED, "1");
+        return pa_source_output_move_to(PA_SOURCE_OUTPUT(obj), PA_SOURCE(parent), false);
+    }
 }
 
-static void move_object_for_filter(pa_object *o, struct filter* filter, bool restore, bool is_sink_input) {
+static void move_object_for_filter(struct userdata *u, pa_object *o, struct filter* filter, bool restore, bool is_sink_input) {
     pa_object *parent;
     pa_proplist *pl;
     const char *name;
@@ -312,7 +312,7 @@ static void move_object_for_filter(pa_object *o, struct filter* filter, bool res
 
     pa_proplist_sets(pl, PA_PROP_FILTER_APPLY_MOVING, "1");
 
-    if (do_move(o, parent, false, is_sink_input) < 0)
+    if (do_move(u, o, parent, is_sink_input) < 0)
         pa_log_info("Failed to move %s for \"%s\" to <%s>.", is_sink_input ? "sink-input" : "source-output",
                     pa_strnull(pa_proplist_gets(pl, PA_PROP_APPLICATION_NAME)), name);
     else
@@ -326,7 +326,7 @@ static void move_objects_for_filter(struct userdata *u, pa_object *o, struct fil
         bool is_sink_input) {
 
     if (!should_group_filter(filter))
-        move_object_for_filter(o, filter, restore, is_sink_input);
+        move_object_for_filter(u, o, filter, restore, is_sink_input);
     else {
         pa_source_output *so;
         pa_sink_input *si;
@@ -339,7 +339,7 @@ static void move_objects_for_filter(struct userdata *u, pa_object *o, struct fil
             g = get_group(PA_OBJECT(so), false);
 
             if (pa_streq(g, group))
-                move_object_for_filter(PA_OBJECT(so), filter, restore, false);
+                move_object_for_filter(u, PA_OBJECT(so), filter, restore, false);
 
             pa_xfree(g);
         }
@@ -348,7 +348,7 @@ static void move_objects_for_filter(struct userdata *u, pa_object *o, struct fil
             g = get_group(PA_OBJECT(si), true);
 
             if (pa_streq(g, group))
-                move_object_for_filter(PA_OBJECT(si), filter, restore, true);
+                move_object_for_filter(u, PA_OBJECT(si), filter, restore, true);
 
             pa_xfree(g);
         }
@@ -367,7 +367,7 @@ static void find_filters_for_module(struct userdata *u, pa_module *m, const char
 
     PA_IDXSET_FOREACH(sink, u->core->sinks, idx) {
         if (sink->module == m) {
-            pa_assert(sink->input_to_master != NULL);
+            pa_assert(pa_sink_is_filter(sink));
 
             fltr = filter_new(name, sink->input_to_master->sink, NULL);
             fltr->module_index = m->index;
@@ -379,7 +379,7 @@ static void find_filters_for_module(struct userdata *u, pa_module *m, const char
 
     PA_IDXSET_FOREACH(source, u->core->sources, idx) {
         if (source->module == m && !source->monitor_of) {
-            pa_assert(source->output_from_master != NULL);
+            pa_assert(pa_source_is_filter(source));
 
             if (!fltr) {
                 fltr = filter_new(name, NULL, source->output_from_master->source);
@@ -416,6 +416,8 @@ static pa_hook_result_t process(struct userdata *u, pa_object *o, bool is_sink_i
     pa_sink *sink = NULL;
     pa_source *source = NULL;
     pa_module *module = NULL;
+    char *module_name = NULL;
+    struct filter *fltr = NULL, *filter = NULL;
 
     if (is_sink_input) {
         sink = PA_SINK_INPUT(o)->sink;
@@ -431,31 +433,27 @@ static pa_hook_result_t process(struct userdata *u, pa_object *o, bool is_sink_i
 
     /* If there is no sink/source yet, we can't do much */
     if ((is_sink_input && !sink) || (!is_sink_input && !source))
-        return PA_HOOK_OK;
+        goto done;
 
     /* If the stream doesn't what any filter, then let it be. */
     if ((want = should_filter(o, is_sink_input))) {
-        char *module_name;
-        struct filter *fltr, *filter;
-
         /* We need to ensure the SI is playing on a sink of this type
          * attached to the sink it's "officially" playing on */
 
         if (!module)
-            return PA_HOOK_OK;
+            goto done;
 
         module_name = pa_sprintf_malloc("module-%s", want);
         if (pa_streq(module->name, module_name)) {
             pa_log_debug("Stream appears to be playing on an appropriate sink already. Ignoring.");
-            pa_xfree(module_name);
-            return PA_HOOK_OK;
+            goto done;
         }
 
         fltr = filter_new(want, sink, source);
 
         if (should_group_filter(fltr) && !find_paired_master(u, fltr, o, is_sink_input)) {
             pa_log_debug("Want group filtering but don't have enough streams.");
-            return PA_HOOK_OK;
+            goto done;
         }
 
         if (!(filter = pa_hashmap_get(u->filters, fltr))) {
@@ -478,14 +476,10 @@ static pa_hook_result_t process(struct userdata *u, pa_object *o, bool is_sink_i
             pa_xfree(args);
         }
 
-        pa_xfree(fltr);
-
         if (!filter) {
             pa_log("Unable to load %s", module_name);
-            pa_xfree(module_name);
-            return PA_HOOK_OK;
+            goto done;
         }
-        pa_xfree(module_name);
 
         /* We can move the stream now as we know the destination. If this
          * isn't true, we will do it later when the sink appears. */
@@ -495,7 +489,6 @@ static pa_hook_result_t process(struct userdata *u, pa_object *o, bool is_sink_i
         }
     } else {
         void *state;
-        struct filter *filter = NULL;
 
         /* We do not want to filter... but are we already filtered?
          * This can happen if an input's proplist changes */
@@ -511,6 +504,10 @@ static pa_hook_result_t process(struct userdata *u, pa_object *o, bool is_sink_i
     if (done_something)
         trigger_housekeeping(u);
 
+done:
+    pa_xfree(module_name);
+    pa_xfree(fltr);
+
     return PA_HOOK_OK;
 }
 
@@ -528,6 +525,9 @@ static pa_hook_result_t sink_input_move_finish_cb(pa_core *core, pa_sink_input *
     if (pa_proplist_gets(i->proplist, PA_PROP_FILTER_APPLY_MOVING))
         return PA_HOOK_OK;
 
+    /* If we're managing m-d-m.auto_filtered on this, remove and re-add if we're continuing to manage it */
+    pa_hashmap_remove(u->mdm_ignored_inputs, i);
+
     return process(u, PA_OBJECT(i), true);
 }
 
@@ -547,6 +547,8 @@ static pa_hook_result_t sink_input_unlink_cb(pa_core *core, pa_sink_input *i, st
     if (pa_hashmap_size(u->filters) > 0)
         trigger_housekeeping(u);
 
+    pa_hashmap_remove(u->mdm_ignored_inputs, i);
+
     return PA_HOOK_OK;
 }
 
@@ -600,6 +602,9 @@ static pa_hook_result_t source_output_move_finish_cb(pa_core *core, pa_source_ou
     if (pa_proplist_gets(o->proplist, PA_PROP_FILTER_APPLY_MOVING))
         return PA_HOOK_OK;
 
+    /* If we're managing m-d-m.auto_filtered on this, remove and re-add if we're continuing to manage it */
+    pa_hashmap_remove(u->mdm_ignored_outputs, o);
+
     return process(u, PA_OBJECT(o), false);
 }
 
@@ -619,6 +624,8 @@ static pa_hook_result_t source_output_unlink_cb(pa_core *core, pa_source_output
     if (pa_hashmap_size(u->filters) > 0)
         trigger_housekeeping(u);
 
+    pa_hashmap_remove(u->mdm_ignored_outputs, o);
+
     return PA_HOOK_OK;
 }
 
@@ -658,6 +665,16 @@ static pa_hook_result_t source_unlink_cb(pa_core *core, pa_source *source, struc
     return PA_HOOK_OK;
 }
 
+static void unset_mdm_ignore_input(pa_sink_input *i)
+{
+    pa_sink_input_set_property(i, PA_PROP_MDM_AUTO_FILTERED, NULL);
+}
+
+static void unset_mdm_ignore_output(pa_source_output *o)
+{
+    pa_source_output_set_property(o, PA_PROP_MDM_AUTO_FILTERED, NULL);
+}
+
 int pa__init(pa_module *m) {
     pa_modargs *ma = NULL;
     struct userdata *u;
@@ -680,17 +697,19 @@ int pa__init(pa_module *m) {
     }
 
     u->filters = pa_hashmap_new(filter_hash, filter_compare);
+    u->mdm_ignored_inputs = pa_hashmap_new_full(NULL, NULL, (pa_free_cb_t) unset_mdm_ignore_input, NULL);
+    u->mdm_ignored_outputs = pa_hashmap_new_full(NULL, NULL, (pa_free_cb_t) unset_mdm_ignore_output, NULL);
 
-    u->sink_input_put_slot = pa_hook_connect(&m->core->hooks[PA_CORE_HOOK_SINK_INPUT_PUT], PA_HOOK_LATE, (pa_hook_cb_t) sink_input_put_cb, u);
-    u->sink_input_move_finish_slot = pa_hook_connect(&m->core->hooks[PA_CORE_HOOK_SINK_INPUT_MOVE_FINISH], PA_HOOK_LATE, (pa_hook_cb_t) sink_input_move_finish_cb, u);
-    u->sink_input_proplist_slot = pa_hook_connect(&m->core->hooks[PA_CORE_HOOK_SINK_INPUT_PROPLIST_CHANGED], PA_HOOK_LATE, (pa_hook_cb_t) sink_input_proplist_cb, u);
-    u->sink_input_unlink_slot = pa_hook_connect(&m->core->hooks[PA_CORE_HOOK_SINK_INPUT_UNLINK], PA_HOOK_LATE, (pa_hook_cb_t) sink_input_unlink_cb, u);
-    u->sink_unlink_slot = pa_hook_connect(&m->core->hooks[PA_CORE_HOOK_SINK_UNLINK], PA_HOOK_LATE-1, (pa_hook_cb_t) sink_unlink_cb, u);
-    u->source_output_put_slot = pa_hook_connect(&m->core->hooks[PA_CORE_HOOK_SOURCE_OUTPUT_PUT], PA_HOOK_LATE, (pa_hook_cb_t) source_output_put_cb, u);
-    u->source_output_move_finish_slot = pa_hook_connect(&m->core->hooks[PA_CORE_HOOK_SOURCE_OUTPUT_MOVE_FINISH], PA_HOOK_LATE, (pa_hook_cb_t) source_output_move_finish_cb, u);
-    u->source_output_proplist_slot = pa_hook_connect(&m->core->hooks[PA_CORE_HOOK_SOURCE_OUTPUT_PROPLIST_CHANGED], PA_HOOK_LATE, (pa_hook_cb_t) source_output_proplist_cb, u);
-    u->source_output_unlink_slot = pa_hook_connect(&m->core->hooks[PA_CORE_HOOK_SOURCE_OUTPUT_UNLINK], PA_HOOK_LATE, (pa_hook_cb_t) source_output_unlink_cb, u);
-    u->source_unlink_slot = pa_hook_connect(&m->core->hooks[PA_CORE_HOOK_SOURCE_UNLINK], PA_HOOK_LATE-1, (pa_hook_cb_t) source_unlink_cb, u);
+    pa_module_hook_connect(m, &m->core->hooks[PA_CORE_HOOK_SINK_INPUT_PUT], PA_HOOK_LATE, (pa_hook_cb_t) sink_input_put_cb, u);
+    pa_module_hook_connect(m, &m->core->hooks[PA_CORE_HOOK_SINK_INPUT_MOVE_FINISH], PA_HOOK_LATE, (pa_hook_cb_t) sink_input_move_finish_cb, u);
+    pa_module_hook_connect(m, &m->core->hooks[PA_CORE_HOOK_SINK_INPUT_PROPLIST_CHANGED], PA_HOOK_LATE, (pa_hook_cb_t) sink_input_proplist_cb, u);
+    pa_module_hook_connect(m, &m->core->hooks[PA_CORE_HOOK_SINK_INPUT_UNLINK], PA_HOOK_LATE, (pa_hook_cb_t) sink_input_unlink_cb, u);
+    pa_module_hook_connect(m, &m->core->hooks[PA_CORE_HOOK_SINK_UNLINK], PA_HOOK_LATE-1, (pa_hook_cb_t) sink_unlink_cb, u);
+    pa_module_hook_connect(m, &m->core->hooks[PA_CORE_HOOK_SOURCE_OUTPUT_PUT], PA_HOOK_LATE, (pa_hook_cb_t) source_output_put_cb, u);
+    pa_module_hook_connect(m, &m->core->hooks[PA_CORE_HOOK_SOURCE_OUTPUT_MOVE_FINISH], PA_HOOK_LATE, (pa_hook_cb_t) source_output_move_finish_cb, u);
+    pa_module_hook_connect(m, &m->core->hooks[PA_CORE_HOOK_SOURCE_OUTPUT_PROPLIST_CHANGED], PA_HOOK_LATE, (pa_hook_cb_t) source_output_proplist_cb, u);
+    pa_module_hook_connect(m, &m->core->hooks[PA_CORE_HOOK_SOURCE_OUTPUT_UNLINK], PA_HOOK_LATE, (pa_hook_cb_t) source_output_unlink_cb, u);
+    pa_module_hook_connect(m, &m->core->hooks[PA_CORE_HOOK_SOURCE_UNLINK], PA_HOOK_LATE-1, (pa_hook_cb_t) source_unlink_cb, u);
 
     pa_modargs_free(ma);
 
@@ -713,27 +732,6 @@ void pa__done(pa_module *m) {
     if (!(u = m->userdata))
         return;
 
-    if (u->sink_input_put_slot)
-        pa_hook_slot_free(u->sink_input_put_slot);
-    if (u->sink_input_move_finish_slot)
-        pa_hook_slot_free(u->sink_input_move_finish_slot);
-    if (u->sink_input_proplist_slot)
-        pa_hook_slot_free(u->sink_input_proplist_slot);
-    if (u->sink_input_unlink_slot)
-        pa_hook_slot_free(u->sink_input_unlink_slot);
-    if (u->sink_unlink_slot)
-        pa_hook_slot_free(u->sink_unlink_slot);
-    if (u->source_output_put_slot)
-        pa_hook_slot_free(u->source_output_put_slot);
-    if (u->source_output_move_finish_slot)
-        pa_hook_slot_free(u->source_output_move_finish_slot);
-    if (u->source_output_proplist_slot)
-        pa_hook_slot_free(u->source_output_proplist_slot);
-    if (u->source_output_unlink_slot)
-        pa_hook_slot_free(u->source_output_unlink_slot);
-    if (u->source_unlink_slot)
-        pa_hook_slot_free(u->source_unlink_slot);
-
     if (u->housekeeping_time_event)
         u->core->mainloop->time_free(u->housekeeping_time_event);
 
@@ -748,5 +746,11 @@ void pa__done(pa_module *m) {
         pa_hashmap_free(u->filters);
     }
 
+    if (u->mdm_ignored_inputs)
+        pa_hashmap_free(u->mdm_ignored_inputs);
+
+    if (u->mdm_ignored_outputs)
+        pa_hashmap_free(u->mdm_ignored_outputs);
+
     pa_xfree(u);
 }
