@@ -14,9 +14,7 @@
   General Public License for more details.
 
   You should have received a copy of the GNU Lesser General Public License
-  along with PulseAudio; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-  USA.
+  along with PulseAudio; if not, see <http://www.gnu.org/licenses/>.
 ***/
 
 #ifdef HAVE_CONFIG_H
@@ -59,7 +57,7 @@ static void help(const char *argv0) {
     printf("%s %s %s\n", argv0, "set-(sink-input|source-output)-mute", _("#N 1|0"));
     printf("%s %s %s\n", argv0, "update-(sink|source)-proplist", _("NAME|#N KEY=VALUE"));
     printf("%s %s %s\n", argv0, "update-(sink-input|source-output)-proplist", _("#N KEY=VALUE"));
-    printf("%s %s %s\n", argv0, "set-default(sink|source)", _("NAME|#N"));
+    printf("%s %s %s\n", argv0, "set-default-(sink|source)", _("NAME|#N"));
     printf("%s %s %s\n", argv0, "kill-(client|sink-input|source-output)", _("#N"));
     printf("%s %s %s\n", argv0, "play-sample", _("NAME SINK|#N"));
     printf("%s %s %s\n", argv0, "remove-sample", _("NAME"));
@@ -75,7 +73,7 @@ static void help(const char *argv0) {
     printf("%s %s %s\n", argv0, "set-(sink|source)-port", _("NAME|#N PORT"));
     printf("%s %s %s\n", argv0, "set-port-latency-offset", _("CARD-NAME|CARD-#N PORT OFFSET"));
     printf("%s %s %s\n", argv0, "set-log-target", _("TARGET"));
-    printf("%s %s %s\n", argv0, "set-log-level", _("NUMERIC LEVEL"));
+    printf("%s %s %s\n", argv0, "set-log-level", _("NUMERIC-LEVEL"));
     printf("%s %s %s\n", argv0, "set-log-meta", _("1|0"));
     printf("%s %s %s\n", argv0, "set-log-time", _("1|0"));
     printf("%s %s %s\n", argv0, "set-log-backtrace", _("FRAMES"));
@@ -333,7 +331,7 @@ int main(int argc, char*argv[]) {
             if (watch_socket->revents & POLLHUP) {
                 ibuf_eof = true;
                 ibuf_length = 0;
-            } if (watch_socket->revents & POLLOUT) {
+            } else if (watch_socket->revents & POLLOUT) {
                 ssize_t r;
                 pa_assert(ibuf_length > 0);
 
